import os
from config import Config
from resources import Resource
from future_fields import FutureField, FutureFields

def get_base_path():
    return os.path.realpath(os.path.dirname(__file__))

def get_test_path():
    return os.path.realpath(
        os.path.join(
            get_base_path(),
            "./test/"
        )
    )

def test_config():
    conf = Config("config", get_test_path)
    conf.define_key("field_1",1)
    conf.define_key("field_2","acc")
    print(conf.read(), conf.data)


def test():
    os.makedirs(get_test_path(), exist_ok=True)
    test_config()

if __name__ == "__main__":
    test()


class FutureFields:
    """
    to use FutureField, you have to initialize a FutureFields
    to store the data of FutureField(s)
    """
    field_containers = {}
    def __init__(self, namespace:str="default") -> None:
        self.field_containers[namespace] = self
        self.fields = {}

class FutureField:
    """
    define a field that work across file
    """
    def __init__(self, name:str, namespace:str="default") -> None:
        self.name = name
        self.namespace = namespace
    
    def set(self, value) -> None:
        """
        set. nothing else
        """
        if not FutureFields.I: raise ValueError("Object 'FutureFields namespace={}' doesn't exists.".format(repr(self.namespace)))
        f = FutureFields.I.fields
        f[self.name] = value

    def get(self):
        """
        get. nothing else
        """
        if not FutureFields.I: raise ValueError("Object 'FutureFields namespace={}' doesn't exists.".format(repr(self.namespace)))
        f = FutureFields.I.fields
        return f.get(self.name, None)


import os
import json
from types import FunctionType

class Config:
    """
    config
    """
    def __init__(self, name:str, base_path:FunctionType) -> None:
        self.name = name
        self.keys = []
        self.data = {}
        self.base_path =base_path()

    def define_key(self, key:str, default: int) -> "Config":
        """
        define a key and set default value
        when the config file doesn't exist
        create a new file and fill it with 
        default values
        """
        self.keys.append(
            (key, default)
        )
        return self

    def get_path(self) -> str:
        """
        get the realpath of the config file
        """
        return os.path.realpath(
            os.path.join(
                self.base_path,
                "./{}.json".format(self.name)
            )
        )
    
    def get_default(self) -> dict:
        """
        get a default config dict
        """
        config = {}
        for key, default in self.keys:
            config[key] = default
        return config

    def read(self) -> bool:
        """
        read local config
        return True if file config exists
        """
        try:
            with open(self.get_path(), "r") as f:
                self.data = json.load(f)
            return True
        except FileNotFoundError:
            """
            create one instead
            """
            self.init()
            return False
    
    def init(self) -> None:
        """
        create a new config
        """
        self.data = self.get_default()
        with open(self.get_path(),"w+") as f:
            json.dump(
                self.data, 
                f, 
                ensure_ascii=False,
                sort_keys=True,
                indent=4
            )


from setuptools import setup

data = {
    "name": "ez-app",
    "version": "1.0.1",
    "author": "TunaFish2K",
    "author_email": "tunafish2k@163.com",
    "description": "app toolkits",
    "long_description": "serveral toolkits with documents that are useful for building an app.",
    "keywords": "app",
    "packages": ["ez-app"],
    "python_requires": ">=3"
}

setup(
    **data
)


import os

class Resource:
    """
    readonly file / folder
    easily create a directory tree
    """
    def __init__(self, where:str, type:str) -> None:
        self.where = where
        self.type = type
        self.name = ""
        self.postfix = ""
        self.children = {}
        self.file = None
    
    def read(self) -> None:
        """
        open the file the resource has
        you still need to read the property 'file'
        of the object by yourself
        """
        if self.type=="file": 
            self.close()
            self.file = open(self.where,"r",encoding="utf-8")
    
    def close(self) -> None:
        """
        close the file the resource has
        """
        if self.file:
            self.file.close()
            self.file = None

    def get(self, name:str) -> "Resource":
        """
        get the file with the specific name under the resource,
        only available for folder,
        return None if not found.
        """
        return self.children.get(name,None)

    @staticmethod
    def create(where:str) -> "Resource":
        """
        create a resource from the specific location,
        automatically create the children under it if
        it is a folder
        """
        r = Resource(where,"file" if os.path.isfile(where) else "folder")
        Resource.create_from_resource(r)
        return r
    
    def __repr__(self) -> str:
        """
        you can understand it, can't you?
        """
        return '<Resource where="{}" type="{}" name="{}" postfix="{}">'.format(self.where,self.type,self.name,self.postfix)

    @staticmethod
    def create_from_resource(where:"Resource") -> None:
        """
        work with a folder Resource
        create all the children under the folder
        """
        for filename in os.listdir(where.where):
            file_path = os.path.realpath( os.path.join(where.where,filename) )
            isfile = os.path.isfile(file_path)

            where.children[filename] = r = Resource(file_path,"file" if isfile else "folder")

            if isfile: 
                name, postfix = filename.split(".",1)
                r.name = name
                r.postfix=postfix  
            
            if not isfile: Resource.create_from_resource(r)
